/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-03-26 17:11:05Z alouis $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component.common.xmlmanipulation;

/**
 * 
 * @author alouis, ofabre - EBM Websourcing
 * 
 */
public final class StringHelper {
    
    private StringHelper() {
    }
    
    /**
     * Test the equality of the specified strings. test is : ( (a==b==null) ||
     * a.equals(b) )
     * 
     * @param a
     * @param b
     * @return
     */
    public static boolean equal(String a, String b) {
        if (a != null) {
            return a.equals(b);
        } else {
            return b == null;
        }
    }

    /**
     * Same as equal, with ignoreCase
     * 
     * @param a
     * @param b
     * @return
     */
    public static boolean equalIgnoreCase(String a, String b) {
        if (a != null) {
            return a.equalsIgnoreCase(b);
        } else {
            return b == null;
        }
    }
    
    /**
     * Return true if the String is null or its size is 0.
     * @param s
     * @return
     */
    public static boolean isEmpty(String s) {
        if (s == null)
            return true;
        
        if (s.trim().equals(""))
            return true;
        
        return false;
    }
    
    /**
     * Extract in the given String the value corresponding 
     * to the given attribute:
     * ".......<ATTRIBUTE>=VALUE<SEPARATOR>......".
     * - not case sensitive
     * - return the first value for the first matching attribute
     * - the separator between ATT and VALUE must be : "="
     * @param string can be null or empty (return null)
     * @param attribute can be null or empty (return null)
     * @param separator (";"," ","&"...) can be null or empty. 
     * in this case, take the end of the String : "ATT=<VALUE etc etc>"
     * @return the found attribute value, null otherwise
     */
    public static String extractValueForAttribute(String string, String attribute,String separator)
    {
        String result = null;
        
        // check string and att are not empty
        if (! isEmpty(string) && ! isEmpty(attribute))
        {
            // find start index
            int start = string.indexOf(attribute);
            
            if(start>=0 && start < string.length())
            {
                start += attribute.length()+1;
                
                if(start < string.length())
                {
                    // find end index
                    int end = 0;
                   
                    if(isEmpty(separator))
                    {
                        // no separator, end = end of string
                        end = string.length();
                    }
                    else
                    {
                        // a separator is specified, find end index
                        end = string.indexOf(separator,start);
                    
                        if (end <0)
                        {
                            end = string.length();
                        }
                    }
                    result = string.substring(start,end);
                }
            }
        }
        return result;
    }
}
