/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentLoggerTest.java 2:27:42 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.logging;

import java.util.logging.Filter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.component.common.logging.mock.MockResourceBundle;

/**
 * Tests of the ComponentLogger
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ComponentLoggerTest extends TestCase {

    public void testAddHandler() {
        Logger logger = EasyMock.createMock(Logger.class);
        Handler handler = EasyMock.createMock(Handler.class);

        logger.addHandler(handler);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.addHandler(handler);
    }

    public void testConfig() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.config("foo");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.config("foo");
    }

    public void testEntering() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.entering("org.objectweb.petals.test", "foo");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.entering("org.objectweb.petals.test", "foo");
    }

    public void testEntering1() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.entering("org.objectweb.petals.test", "foo", "object1");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.entering("org.objectweb.petals.test", "foo", "object1");
    }

    public void testEntering2() {
        Object[] objs = new Object[0];
        Logger logger = EasyMock.createMock(Logger.class);

        logger.entering("org.objectweb.petals.test", "foo", objs);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.entering("org.objectweb.petals.test", "foo", objs);
    }

    public void testEquals() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.equals(logger);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertTrue(componentLogger.equals(logger));
    }

    public void testExiting() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.exiting("org.objectweb.petals.test", "foo");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.exiting("org.objectweb.petals.test", "foo");
    }

    public void testExiting1() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.exiting("org.objectweb.petals.test", "foo", "out1");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.exiting("org.objectweb.petals.test", "foo", "out1");
    }

    public void testFine() {
        Logger logger = EasyMock.createMock(Logger.class);
        MockResourceBundle resourceBundle = new MockResourceBundle();

        EasyMock.expect(logger.getResourceBundle()).andReturn(resourceBundle)
            .anyTimes();
        logger.fine("[compo-test] test msg");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.fine("test");
    }

    public void testFiner() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.finer("[compo-test] test");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.finer("test");
    }

    public void testFinest() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.finest("[compo-test] test");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.finest("test");
    }

    public void testGetFilter() {
        Logger logger = EasyMock.createMock(Logger.class);
        Filter filter = EasyMock.createMock(Filter.class);

        EasyMock.expect(logger.getFilter()).andReturn(filter);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.getFilter(), filter);
    }

    public void testGetHandlers() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getHandlers()).andReturn(new Handler[0]);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.getHandlers().length, 0);
    }

    public void testGetLevel() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getLevel()).andReturn(Level.ALL);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.getLevel(), Level.ALL);
    }

    public void testGetName() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getName()).andReturn("test");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.getName(), "test");
    }

    public void testGetParent() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getParent()).andReturn(null);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.getParent(), null);
    }

    public void testGetResourceBundle() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.getResourceBundle(), null);
    }

    public void testGetResourceBundleName() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundleName()).andReturn(null);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.getResourceBundleName(), null);
    }

    public void testGetUseParentHandlers() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getUseParentHandlers()).andReturn(false);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertFalse(componentLogger.getUseParentHandlers());
    }

    public void testHashCode() {
        Logger logger = EasyMock.createMock(Logger.class);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.hashCode(), logger.hashCode());
    }

    public void testInfo() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();

        logger.info("[compo-test] bar");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.info("bar");
    }

    public void testIsLoggable() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.isLoggable(Level.FINE)).andReturn(true);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertTrue(componentLogger.isLoggable(Level.FINE));
    }

    public void testLog() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.log(Level.CONFIG, "[compo-test] test log");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.log(Level.CONFIG, "test log");
    }

    public void testLogp() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logp(Level.CONFIG, "class", "method", "[compo-test] test log");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logp(Level.CONFIG, "class", "method", "test log");
    }

    public void testLogrb() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logrb(Level.CONFIG, "class", "method", "bundle",
            "[compo-test] test log");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logrb(Level.CONFIG, "class", "method", "bundle",
            "test log");
    }

    public void testLog1() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.log(Level.CONFIG, "[compo-test] test log", "param1");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.log(Level.CONFIG, "test log", "param1");
    }

    public void testLogp1() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logp(Level.CONFIG, "class", "method", "[compo-test] test log",
            "param1");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logp(Level.CONFIG, "class", "method", "test log",
            "param1");
    }

    public void testLogrb1() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logrb(Level.CONFIG, "class", "method", "bundle",
            "[compo-test] test log", "param1");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logrb(Level.CONFIG, "class", "method", "bundle",
            "test log", "param1");
    }

    public void testLog2() {
        Object[] params = new Object[0];
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.log(Level.CONFIG, "[compo-test] test log", params);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.log(Level.CONFIG, "test log", params);
    }

    public void testLogp2() {
        Object[] params = new Object[0];
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logp(Level.CONFIG, "class", "method", "[compo-test] test log",
            params);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logp(Level.CONFIG, "class", "method", "test log",
            params);
    }

    public void testLogrb2() {
        Object[] params = new Object[0];
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logrb(Level.CONFIG, "class", "method", "bundle",
            "[compo-test] test log", params);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logrb(Level.CONFIG, "class", "method", "bundle",
            "test log", params);
    }

    public void testLog3() {
        Exception exception = new Exception();
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.log(Level.CONFIG, "[compo-test] test log", exception);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.log(Level.CONFIG, "test log", exception);
    }

    public void testLogp3() {
        Exception exception = new Exception();
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logp(Level.CONFIG, "class", "method", "[compo-test] test log",
            exception);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logp(Level.CONFIG, "class", "method", "test log",
            exception);
    }

    public void testLogrb3() {
        Exception exception = new Exception();
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.logrb(Level.CONFIG, "class", "method", "bundle",
            "[compo-test] test log", exception);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.logrb(Level.CONFIG, "class", "method", "bundle",
            "test log", exception);
    }

    public void testLog4() {
        Logger logger = EasyMock.createMock(Logger.class);
        LogRecord logRecord = new LogRecord(Level.FINEST, "test");

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();
        logger.log(logRecord);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.log(logRecord);
    }

    public void testRemoveHandler() {
        Logger logger = EasyMock.createMock(Logger.class);
        Handler handler = EasyMock.createMock(Handler.class);

        logger.removeHandler(handler);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.removeHandler(handler);
    }

    public void testSetFilter() {
        Logger logger = EasyMock.createMock(Logger.class);
        Filter filter = EasyMock.createMock(Filter.class);

        logger.setFilter(filter);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.setFilter(filter);
    }

    public void testSetLevel() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.setLevel(Level.SEVERE);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.setLevel(Level.SEVERE);
    }

    public void testSetParent() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.setParent(null);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.setParent(null);
    }

    public void testSetUseParentHandlers() {
        Logger logger = EasyMock.createMock(Logger.class);

        logger.setUseParentHandlers(false);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.setUseParentHandlers(false);
    }

    public void testSevere() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();

        logger.severe("[compo-test] bar");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.severe("bar");
    }

    public void testThrowing() {
        Exception exception = new Exception();
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();

        logger.throwing("source", "method", exception);

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.throwing("source", "method", exception);
    }
    
    public void testToString() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        assertEquals(componentLogger.toString(), logger.toString());
    }
    
    public void testWarning() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getResourceBundle()).andReturn(null).anyTimes();

        logger.warning("[compo-test] bar");

        EasyMock.replay(logger);

        ComponentLogger componentLogger = new ComponentLogger(logger, "test",
            null, "compo-test");

        componentLogger.warning("bar");
    }

}
