/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : FileTransferBC.java 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.binding.filetransferbc.listeners.FileTransferBCJBIListener;
import org.objectweb.petals.binding.filetransferbc.listeners.FileTransferBCListener;
import org.objectweb.petals.component.common.logging.ComponentLogger;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author alouis, Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class FileTransferBC implements Component, ComponentLifeCycle {

    protected ComponentContext context;

    protected Logger logger;

    protected PetalsServiceUnitManager suManager;

    private DeliveryChannel channel;

    private FileTransferBCJBIListener jbiListener;

    private FileTransferBCListener listener;

    private Map<String, Properties> mapDirConfig;

    private FileTransferConfigReceiverSUHandler serviceUnitHandler;

    private String workDir;

    public ObjectName getExtensionMBeanName() {
        return null;
    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    public Document getServiceDescription(ServiceEndpoint arg0) {
        return suManager.getServiceDescription(arg0);
    }

    public ServiceUnitManager getServiceUnitManager() {
        return this.suManager;
    }

    public void init(ComponentContext context) throws JBIException {

        try {

            this.context = context;
            Logger log = context.getLogger("", null);
            logger = new ComponentLogger(log, log.getName(), log
                .getResourceBundleName(), context.getComponentName());
            this.channel = context.getDeliveryChannel();
            this.workDir = getDirectoryRoot();
            this.mapDirConfig = new HashMap<String, Properties>();
            suManager = createServiceUnitManager();
            this.serviceUnitHandler = new FileTransferConfigReceiverSUHandler(
                context, this.logger, this.mapDirConfig, this.workDir);
            suManager.addNewHandler(this.serviceUnitHandler);

            this.jbiListener = createFileTransferBCJBIListener();
            this.listener = createFileTransferBCListener();

        } catch (Exception e) {
            throw new JBIException(e);
        }
        logger.log(Level.INFO, "init");
    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return true;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return true;
    }

    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;
        if (arg0 != null) {
            String prefix = arg0.getPrefix();
            if (prefix == null) {
                prefix = new String();
            } else {
                prefix += ":";
            }
            Node service = XMLHelper.findChild(arg0, prefix + "service", true);

            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                    "name");
                if (serviceRequested != null) {
                    result = suManager.findEndpointForService(QName
                        .valueOf(serviceRequested));
                }
            }
        }
        return result;
    }

    public void shutDown() throws JBIException {
        logger.log(Level.INFO, "shutDown");
    }

    public void start() throws JBIException {
        logger.log(Level.INFO, "start");

        Thread jbiListenerThread = new Thread(this.jbiListener,context.getComponentName()+"-JBI listener thread");
        jbiListenerThread.start();

        Thread listenerThread = new Thread(this.listener,context.getComponentName()+"-directory listener thread");
        listenerThread.start();

    }

    public void stop() throws JBIException {
        logger.log(Level.INFO, "stop");
        jbiListener.stopProcessing();
        listener.stopProcessing();
    }

    /**
     * Create listener of JBI container
     * 
     * @return
     */
    protected FileTransferBCJBIListener createFileTransferBCJBIListener() {
        return new FileTransferBCJBIListener(this.channel, logger,
            this.context, this.workDir);
    }

    /**
     * Create listener of server mail
     * 
     * @return
     */
    protected FileTransferBCListener createFileTransferBCListener() {
        return new FileTransferBCListener(this.logger, this.channel,
            this.context, this.jbiListener, this.mapDirConfig, this.workDir);
    }

    /**
     * Function "create" to simplify test
     * 
     * @return
     * @throws DeploymentException
     */
    protected PetalsServiceUnitManager createServiceUnitManager()
        throws DeploymentException {
        return new PetalsServiceUnitManager(context, logger);
    }

    protected String getDirectoryRoot() {
        String baseDir = context.getInstallRoot();
        baseDir = baseDir.substring(0, baseDir.length() - 7);
        baseDir += "work" + File.separator;
        return baseDir;
    }

}
