/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;

import org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.ManagementMessageUtil;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

public class FileTransferConfigReceiverSUHandler extends
    PetalsServiceUnitHandler {

    protected Map<String, String> mapDirSU;

    private Map<String, Properties> mapDirConfig;

    protected String workDir;

    public FileTransferConfigReceiverSUHandler() {
        super();
        this.mapDirSU = new HashMap<String, String>();
        SERVICE_UNIT_TYPE = "configFileTransfer_services";
    }

    public FileTransferConfigReceiverSUHandler(ComponentContext context,
        Logger logger, Map<String, Properties> mapDirConfig, String dir) {
        super(context, logger);
        this.context = context;
        this.logger = logger;
        this.mapDirConfig = mapDirConfig;
        this.mapDirSU = new HashMap<String, String>();
        this.workDir = dir;
        SERVICE_UNIT_TYPE = "configFileTransfer_services";
    }

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#deploy(java.lang.String, java.lang.String, java.lang.String)
     */
    public String deploy(String serviceUnitName, String serviceUnitType,
        String serviceUnitRootPath) {
        if (SERVICE_UNIT_TYPE.equals(serviceUnitType)) {
            logger.info(" DEPLOY  serviceUnitName  " + serviceUnitName
                + " serviceUnitRootPath" + serviceUnitRootPath);
            return ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "deploy",
                ManagementMessageUtil.TASK_RESULT_SUCCESS);
        } else {
            return null;
        }
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        this.serviceUnitInstallationRootPath.put(serviceUnitName,
            serviceUnitRootPath);
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        return ManagementMessageUtil.getComponentTaskResult(context
            .getComponentName(), "undeploy",
            ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    public void shutDown(String serviceUnitName,
        PetalsServiceUnitManager epHandler) throws DeploymentException {
        String directory = this.mapDirSU.remove(serviceUnitName);
        this.mapDirConfig.remove(directory);
    }

    public void start(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        String serviceUnitRootPath = serviceUnitInstallationRootPath
            .get(serviceUnitName);
        /*
         * Map to save directory -> directory properties And Map to save
         * serviceUnit name and directory, this map is used to stop and
         * shutdown serviceUnit
         */
        Properties props = loadDirectoryConfig(new File(serviceUnitRootPath
            + File.separator + "META-INF" + File.separator + "jbi.xml"));
        String directory = props.getProperty("directory.name");
        if (!directory.equals("")) {
            File file = new File(this.workDir + "upload" + File.separator
                + directory);
            file.mkdirs();

            this.mapDirConfig.put(directory, props);
            this.mapDirSU.put(serviceUnitName, directory);
        } else {
            logger.log(Level.WARNING, "Any directory has been specified");
        }
    }

    public void stop(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        /*
         * When service unit is stopped, so properties of directory are deleted
         */
        String directory = this.mapDirSU.remove(serviceUnitName);
        this.mapDirConfig.remove(directory);
    }

    /**
     * jbiXML define attibuts to configure files to allow echange files.
     * @param jbiXml
     * @return properties
     * properties has all attributs to call an internal Petals service, 
     * the source directory, destination directory, service and method to be called
     */
    protected Properties loadDirectoryConfig(File jbiXml) {
        Properties configProperties = new Properties();
        Document jbiDoc = WSDLHelper.createDocumentFromWSDL(jbiXml);
        Node node = XMLHelper.findChild(jbiDoc, "services", true);
        if (node != null) {

            Node providesNode = XMLHelper.findChild(node, "provides", true);
            if (providesNode != null) {
                Node directoryNode = XMLHelper.findChild(node,
                    "uploadDirectory", true);

                if (directoryNode != null) {
                    String value = directoryNode.getTextContent();
                    configProperties.put("directory.name", value);
                }
            }

            Node consumesNode = XMLHelper.findChild(node, "consumes", true);
            if (consumesNode != null) {

                String service = XMLHelper.getAttributeValue(consumesNode,
                    "service-name");
                /*
                 * if attibut service is not defined, so properties is save like empty String
                 */
                if (service == null) {
                    service = new String("");
                }
                configProperties.put("service.name", service);

                String method = XMLHelper.getAttributeValue(consumesNode,
                    "method-name");
                if (method == null) {
                    method = new String("");
                }
                configProperties.put("operation.name", method);

                Node downloadDirNode = XMLHelper.findChild(node,
                    "donwloadDirectory", true);

                if (downloadDirNode != null) {
                    String value = downloadDirNode.getTextContent();
                    configProperties.put("directory.destination", value);
                }
            }

        }
        return configProperties;
    }
}
