/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import javax.jbi.management.DeploymentException;

import org.easymock.classextension.EasyMock;

import junit.framework.TestCase;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class SUHandlerTest extends TestCase {

    private String baseDir;

    Logger logger;

    @Override
    public void setUp() throws Exception {

        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        logger = EasyMock.createMock(Logger.class);
        super.setUp();
    }

    public void testShutdown() {

        Map<String, Properties> match = new HashMap<String, Properties>(); //
        // insert a pair in map
        match.put("directoryName", new Properties());
        String dir = baseDir + "src" + File.separator + "test-data"
            + File.separator;
        String upload = dir + "directory" + File.separator;
        FileTransferConfigReceiverSUHandler handler = new FileTransferConfigReceiverSUHandler(
            null, logger, match, upload);
        handler.mapDirSU.put("serviceUnit", "directoryName");
        try {
            handler.shutDown("serviceUnit", null);
        } catch (DeploymentException e) {
            e.printStackTrace();
        }
        assertFalse(handler.mapDirSU.containsKey("serviceUnit"));
        assertFalse(match.containsKey("directoryName"));

    }

    public void testStart() {

        Map<String, Properties> match = new HashMap<String, Properties>();
        String dir = baseDir + "src" + File.separator + "test-data"
            + File.separator;
        String upload = dir + "directory" + File.separator;
        FileTransferConfigReceiverSUHandler handler = new FileTransferConfigReceiverSUHandler(
            null, logger, match, upload) {

            protected Properties loadDirectoryConfig(File jbiXml) {
                Properties props = new Properties();
                props.put("directory.name", "test");
                return props;
            }

        };
        handler.workDir = upload;
        // File jbiXml= new File("META-INF"+File.separator+"jbi.xml");

        try {
            /*
             * Init allows initialize the Map serviceUnitInstallationRootPath
             * that match
             * 
             */
            handler.init("serviceUnitName", upload);

            handler.start("serviceUnitName", null);
        } catch (DeploymentException e) {
            e.printStackTrace();
        }

        boolean createSUDir = false;
        File file = new File(upload);
        File[] list = file.listFiles();
        for (File file2 : list) {
            if (("upload").equals(file2.getName())) {
                File[] direc = file2.listFiles();
                for (File file3 : direc) {
                    if (("test").equals(file3.getName())) {
                        createSUDir = true;
                    }
                    file3.delete();
                }
            }
            file2.delete();
        }
        file.delete();
        assertTrue(createSUDir);
        assertTrue(match.containsKey("test"));
        match.clear();
    }

    public void testStop() {

        Map<String, Properties> match = new HashMap<String, Properties>(); //
        // insert a pair in map match.put("directoryName", new Properties());
        String dir = baseDir + "src" + File.separator + "test-data"
            + File.separator;
        String upload = dir + "directory" + File.separator;
        FileTransferConfigReceiverSUHandler handler = new FileTransferConfigReceiverSUHandler(
            null, this.logger, match, upload);

        handler.mapDirSU.put("serviceUnit", "directoryName");
        try {
            handler.stop("serviceUnit", null);
        } catch (DeploymentException e) {
            e.printStackTrace();
        }
        assertFalse(handler.mapDirSU.containsKey("serviceUnit"));
        assertFalse(match.containsKey("directoryName"));
    }

    public void testLoadDirectoryConfig() {
        String dir = baseDir + "src" + File.separator + "test-data"
            + File.separator + "jbiServiceUnit.xml";

        File jbiXml = new File(dir);
        Properties props = new Properties();
        props.put("directory.name", "directory");
        props.put("operation.name", "method");
        props.put("directory.destination", "destination");
        props.put("service.name", "{http://test.org/}Service");

        FileTransferConfigReceiverSUHandler handler = new FileTransferConfigReceiverSUHandler(
            null, null, null, null);

        Properties propsResult = handler.loadDirectoryConfig(jbiXml);

        assertTrue(props.equals(propsResult));
    }
}
