/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc.listeners;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.binding.filetransferbc.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.filetransferbc.mock.MockFileTransferBCJBIListener;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.w3c.dom.Document;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class FileTransferBCListenerTest extends TestCase {
    String baseDir;

    Logger logger;

    String workDir;

    private ComponentContext context;

    private MockDeliveryChannel channel;

    @Override
    public void setUp() throws Exception {
        super.setUp();
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        workDir = baseDir + "src" + File.separator + "test-data"
            + File.separator;
        this.logger = EasyMock.createMock(Logger.class);

        channel = new MockDeliveryChannel();

        context = createMock(ComponentContext.class);
        expect(context.getDeliveryChannel()).andReturn(channel).anyTimes();
        replay(context);
    }

    public void testCreateWorkDirectory() {
        boolean created = false;
        FileTransferBCListener listener = new FileTransferBCListener(
            this.logger, null, null, null, null, workDir);

        listener.createWorkDirectory();

        File[] files = new File(workDir).listFiles();

        for (File file : files) {
            if (("upload").equals(file.getName())) {
                File[] files2 = file.listFiles();
                for (File file2 : files2) {
                    if (("waste").equals(file2.getName())) {
                        created = true;
                    }
                    file2.delete();
                }
                file.delete();
            }
        }
        assertTrue(created);
    }

    public void testHandleInOnly() throws MessagingException {

        FileTransferBCListener listener = new FileTransferBCListener(
            this.logger, channel, context, null, null, "dir");
        try {
            listener.handleInOnlyMessage("body", QName.valueOf("service"),
                "operation");
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        }
        verify(context);
        assertTrue(channel.isSendSync());
    }

    public void testHandleRobustInOnly() throws MessagingException {

        FileTransferBCListener listener = new FileTransferBCListener(
            this.logger, channel, context, null, null, "dir");

        try {
            listener.handleRobustInOnlyMessage("body",
                QName.valueOf("service"), "operation", "dir");
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        }
        verify(context);
        assertTrue(channel.isSendSync());
    }

    public void testHandleInOut() throws MessagingException {

        FileTransferBCJBIListener jbiList = new FileTransferBCJBIListener(
            channel, this.logger, context, "dir") {
            public boolean writeAttachments(Set attachId, NormalizedMessage nm,
                String subDir) {
                return true;
            }

            public boolean writeFile(String body, String subDir, String fileName) {
                return true;
            }
        };

        FileTransferBCListener listener = new FileTransferBCListener(
            this.logger, channel, context, null, null, "dir");
        listener.listenerJBI = jbiList;

        try {
            listener.handleInOutMessage("body", QName.valueOf("service"),
                "operation", "dir");
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        }
        verify(context);
    }

    public void testHandleNewFiles() {

        Method method1 = null;
        Method method2 = null;

        try {

            method1 = FileTransferBCListener.class.getDeclaredMethod(
                "handleInOnlyMessage", new Class[] {String.class, QName.class,
                    String.class});
            method2 = FileTransferBCListener.class.getDeclaredMethod(
                "handleInOutMessage", new Class[] {String.class, QName.class,
                    String.class, String.class});

        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1, method2};
        FileTransferBCListener listener = createMock(
            FileTransferBCListener.class, mockedMethods);
        listener.attachments = new HashSet<DataHandler>();
        // Create directory with files to be sents
        File file = new File(workDir + "test");
        File waste = new File(workDir + "waste");
        file.mkdirs();
        waste.mkdirs();
        File text1 = new File(file.getAbsoluteFile() + File.separator
            + "text1.txt");
        File text2 = new File(file.getAbsoluteFile() + File.separator
            + "text2.txt");

        try {
            text1.createNewFile();
            text2.createNewFile();
        } catch (IOException e) {
            e.printStackTrace();
        }
        String body = "<?xml version='1.0' encoding='UTF-8'?><info>attached file(s)</info>";
        try {
            expect(
                listener.handleInOnlyMessage(body, QName.valueOf("service"),
                    "operation")).andReturn(true);
            expect(
                listener.handleInOutMessage(body, QName.valueOf("service"),
                    "operation", "testResp" + File.separator)).andReturn(true);
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        }
        replay(listener);

        boolean inOut = true;
        // it is an inOut operation
        listener.handleNewFiles(file, inOut, QName.valueOf("service"),
            "operation", "testResp" + File.separator);

        // it is an inOnly operation
        inOut = false;
        listener.handleNewFiles(file, inOut, QName.valueOf("service"),
            "operation", "test" + File.separator);

        boolean moved = true;
        File[] files = file.listFiles();
        for (File file2 : files) {
            if (("text1.txt").equals(file2.getName())
                || ("text2.txt").equals(file2.getName())) {
                moved = false;
            }
        }
        file.delete();
        assertTrue(moved);
        verify(listener);
        files = waste.listFiles();
        for (File file2 : files) {
            file2.delete();
        }
        waste.delete();
    }

    public void testNotFoundService() {

        Method method1 = null;

        try {

            method1 = FileTransferBCJBIListener.class.getDeclaredMethod(
                "writeFile", new Class[] {String.class, String.class,
                    String.class});

        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};
        FileTransferBCListener listener = new FileTransferBCListener(
            this.logger, null, null, null, null, baseDir);

        FileTransferBCJBIListener jbiList = createMock(
            FileTransferBCJBIListener.class, mockedMethods);
        jbiList.responseDirName = workDir;
        listener.listenerJBI = jbiList;
        String response = SourceHelper.createSoapFault(
            new FileTransferBCException("Service unknown"), "300");
        try {
            expect(jbiList.writeFile(response, "subDir", "log.txt")).andReturn(
                true);
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        }
        replay(jbiList);

        listener.notFoundService(response, "subDir");

        verify(jbiList);
    }

    public void testProcess() {

        FileTransferBCListener listener = null;

        File file = new File(workDir + "fileTransfer.wsdl");
        Document wsdl = WSDLHelper.createDocumentFromWSDL(file);

        ServiceEndpoint endpoint = createMock(ServiceEndpoint.class);

        ServiceEndpoint[] serv = {endpoint};

        ComponentContext context = createMock(ComponentContext.class);
        try {
            expect(endpoint.getServiceName()).andReturn(
                new QName("serviceName"));
            expect(context.getEndpointsForService(new QName("MockService")))
                .andReturn(serv);
            expect(context.getEndpointDescriptor(endpoint)).andReturn(wsdl)
                .anyTimes();
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(endpoint);
        replay(context);

        Map<String, Properties> mapDirConfig = new HashMap<String, Properties>();
        Properties props = new Properties();
        props.put("service.name", "MockService");
        props.put("operation.name", "inOnlyMessage");
        props.put("directory.destination", "destinationDir");
        mapDirConfig.put("MockService", props);
        listener = new FileTransferBCListener(this.logger, null, context, null,
            mapDirConfig, "dir") {
            public void handleNewFiles(File file, boolean inOut, QName service,
                String operation, String dirDest) {

            }
        };

        /*
         * Test case 1: operation inOnly
         */
        MockFileTransferBCJBIListener fileTransferBCJBIListener = new MockFileTransferBCJBIListener(
            null, null, null, null);
        listener.listenerJBI = fileTransferBCJBIListener;

        listener.process(new File("MockService"));
        EasyMock.reset(context);
        EasyMock.reset(endpoint);
        mapDirConfig.clear();
        assertTrue(fileTransferBCJBIListener.isWriteFile());

        /*
         * Test case 2 :operation inOut
         */

        try {
            expect(endpoint.getServiceName()).andReturn(
                new QName("serviceName"));
            expect(context.getEndpointsForService(new QName("MockService")))
                .andReturn(serv);
            expect(context.getEndpointDescriptor(endpoint)).andReturn(wsdl)
                .anyTimes();
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(endpoint);
        replay(context);

        props.put("operation.name", "inOutOperation");
        mapDirConfig.put("MockService", props);
        listener.process(new File("MockService"));

        verify(context);
        verify(endpoint);
    }
}
