/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: HelloworldProcessor.java 12:18:02 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.helloworld;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.NormalizedMessage;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.listener.AbstractInternalMEProcessor;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;

/**
 * This class is the message processor for the HelloworldImpl component
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 * @author ddesjardins - eBMWebsourcing
 * @author ofabre - eBMWebsourcing
 */
public class HelloworldProcessor extends AbstractInternalMEProcessor {

    protected InitialContext componentInitialContext;

    protected Helloworld helloworld;

    public HelloworldProcessor(DeliveryChannel c, Logger l,
        Helloworld helloworld) {

        super(c, l);

        this.helloworld = helloworld;
    }

    @Override
    protected void sendInMessage(QName service, QName operation,
        NormalizedMessage in) throws Exception {

        if (new QName("printMessage").equals(operation)) {

            String content = SourceHelper.createString(in.getContent());

            helloworld.printMessage(content);

        } else if (new QName("bindEntryInComponentInitialContext")
            .equals(operation)) {

            String content = SourceHelper.createString(in.getContent());

            helloworld.bindMessage(content);

        } else {

            l.log(Level.WARNING, "HelloworldImpl : incompatible operation/MEP");

            // this will be automaticly added to a Fault if the message is
            // Robust InOnly
            throw new Exception(
                "Only 'printMessage' or 'bindMessage' operations "
                    + "are recognized for an InOnly message");
        }
    }

    @Override
    protected boolean sendInOutMessage(QName service, QName operation,
        NormalizedMessage in, NormalizedMessage out, boolean optionalOut)
        throws Exception {

        if (new QName("sayHello").equals(operation)) {

            String content = SourceHelper.createString(in.getContent());

            String response = helloworld.sayHello(content);

            out.setContent(SourceHelper.createSource(response));

            return true;

        } else {

            l.log(Level.WARNING, "hello.incomp");

            // this will be automaticly added to a Fault if the message is
            // Robust InOnly
            throw new Exception(
                "Only 'sayHello' operation is recognized for an InOut message");
        }
    }
}
