/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JbiConnexionFactory.java 601 2006-06-13 12:53:50Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.jms;

import java.io.File;

import java.util.List;

import java.util.logging.Logger;

import org.objectweb.petals.binding.jms.incoming.JbiConnection;
import org.objectweb.petals.tools.jbicommon.descriptor.Consumes;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.tools.jbicommon.descriptor.Provides;
import org.objectweb.petals.tools.jbicommon.descriptor.Services;
import org.xml.sax.SAXException;

/**
 * This class is used to create <code>JbiConnexion</code> object.
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class JbiConnectionFactory {

    protected static JbiConnectionFactory instance = new JbiConnectionFactory();

    protected JbiConnectionFactory() {
    }

    public static JbiConnectionFactory instance() {
        return instance;
    }

    /**
     * Create a JbiConnection from the jbi.xml file,
     * by getting the first services/consumes element.
     * @param serviceUnitRootPath
     * @return a jbiConnection, null if no 'consumes' found
     * @throws SAXException Can not analyse jbi.xml file
     */
    public JbiConnection createConsumesConnection(String serviceUnitRootPath,
        Logger log) throws SAXException {
        JbiConnection conn = null;

        File jbiXmlFile = new File(serviceUnitRootPath + File.separator
            + "META-INF" + File.separator + "jbi.xml");

        // 1. get services/consumes' element 
        JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
            jbiXmlFile.toURI(), log);
        JBIDescriptor descriptor = descriptorBuilder.build();
        Services services = descriptor.getServices();

        List<Consumes> consumesList = services.getConsumes();

        // 2. create a jbiconnection
        if (consumesList != null && consumesList.size() > 0) {
            Consumes consumes = consumesList.get(0);
            conn = new JbiConnection(consumes.getServiceName(), consumes
                .getEndpointName(), consumes.getInterfaceName());
        }
        return conn;
    }

    /**
     * Create a JbiConnection from the jbi.xml file,
     * by getting the first services/provides element.
     * @param serviceUnitRootPath
     * @return a jbiConnection, null if no 'provides' found
     * @throws SAXException Can not analyse jbi.xml file
     */
    public JbiConnection createProvidesConnection(String serviceUnitRootPath,
        Logger log) throws SAXException {
        JbiConnection conn = null;

        File jbiXmlFile = new File(serviceUnitRootPath + File.separator
            + "META-INF" + File.separator + "jbi.xml");

        // 1. get services/consumes' element 
        JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
            jbiXmlFile.toURI(), log);
        JBIDescriptor descriptor = descriptorBuilder.build();
        Services services = descriptor.getServices();

        List<Provides> providesList = services.getProvides();

        // 2. create a jbiconnection
        if (providesList != null && providesList.size() > 0) {
            Provides provides = providesList.get(0);
            conn = new JbiConnection(provides.getServiceName(), provides
                .getEndpointName(), provides.getInterfaceName());
        }
        return conn;
    }

}
