/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:  SMTPServiceUnitManager.java 154 2006-03-27 15:30:10Z rbarraza $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.smtpbc.listeners;

import java.security.cert.X509Certificate;
import java.util.Set;

import javax.activation.DataHandler;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import org.ebmwebsourcing.b2bgw.security.beans.CertificatePKCS12;
import org.objectweb.petals.binding.exception.SMTPBCException;

import com.isnetworks.smime.Capabilities;
import com.isnetworks.smime.EnvelopedBodyPart;
import com.isnetworks.smime.SignedMultipart;

public final class MessageMime {

    private MessageMime() {
        super();
    }

    public static void setAttachments(Multipart multipart,
        Set<DataHandler> attachments) throws SMTPBCException {
        MimeBodyPart messageBodyPart = null;
        for (DataHandler handler : attachments) {
            messageBodyPart = new MimeBodyPart();
            try {
                messageBodyPart.setDataHandler(handler);
                messageBodyPart.setFileName(handler.getName());
                multipart.addBodyPart(messageBodyPart);
            } catch (MessagingException e) {
                throw new SMTPBCException("Error attaching file");
            }
        }

    }

    public static MimeMessage signedAndEnvelope(MimeMessage message,
        CertificatePKCS12 senderCert, X509Certificate caCertificate,
        X509Certificate recipientCert, String body,
        Set<DataHandler> attachedFiles) {
        try {
            // Creating the SignedMultipart and setting the message's content
            MimeBodyPart bodyPart = new MimeBodyPart();
            bodyPart.setContent(body, "text/plain");
            SignedMultipart signedMultipart = new SignedMultipart(bodyPart);

            // add sender certificate to the message
            signedMultipart.addSigner(senderCert.getPrivateKey(), senderCert
                .getCert(), "SHA1withRSA");
            signedMultipart.addCertificate(caCertificate);

            // attach file to mime message, each attachment is in one BodyPart
            if (attachedFiles != null && !attachedFiles.isEmpty()) {
                setAttachments(signedMultipart, attachedFiles);
            }

            // Creating the EnvelopedBodyPart around the SignedMultipart
            EnvelopedBodyPart envelopedPart = new EnvelopedBodyPart(
                signedMultipart);

            // Set to use RC2, 40 bit encryption so that weak crypto clients can
            // read them
            envelopedPart.addRecipient(recipientCert, new Capabilities(false,
                false, false, false, true));

            message.setDataHandler(envelopedPart.getDataHandler());
            message.setDisposition("attachment; filename=smime.p7m");
            message.setDescription("S/MIME Encrypted Message");
            return message;

        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }

    public static MimeMessage signedMultipart(MimeMessage message,
        CertificatePKCS12 senderCert, X509Certificate caCertificate,
        String body, Set<DataHandler> attachedFiles) {
        try {
            MimeBodyPart bodyPart = new MimeBodyPart();

            bodyPart.setContent(body, "text/plain");
            SignedMultipart signedMultipart = new SignedMultipart(bodyPart);
            signedMultipart.addSigner(senderCert.getPrivateKey(), senderCert
                .getCert(), "MD5withRSA");
            signedMultipart.addCertificate(caCertificate);

            // attach file to mime message
            if (attachedFiles != null && !attachedFiles.isEmpty()) {
                setAttachments(signedMultipart, attachedFiles);
            }
            message.setContent(signedMultipart);
            message.setDisposition("attachment; filename=smime.p7m");
            message.setDescription("S/MIME Signed Message");
            return message;

        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }

    public static MimeMessage simpleMessage(MimeMessage message,
        String msgBody, Set<DataHandler> attachedFiles) {
        try {
            MimeBodyPart messageBodyPart = new MimeBodyPart();
            messageBodyPart.setContent(msgBody, "text/plain");
            Multipart multipart = new MimeMultipart();
            multipart.addBodyPart(messageBodyPart);

            if (attachedFiles != null && !attachedFiles.isEmpty()) {
                setAttachments(multipart, attachedFiles);
            }
            message.setContent(multipart);
            return message;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }
}
