/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:  SMTPServiceUnitManager.java 154 2006-03-27 15:30:10Z rbarraza $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.smtpbc.listeners;

/**
 * 
 * This listener listen messages send by JBI container, and send an e-mail to
 * External Endpoint that offert the service.
 * 
 * @author Rocio BARRAZA - eBMWEBSOURCING
 * 
 */

import java.io.File;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.ebmwebsourcing.b2bgw.security.beans.CertificatePKCS12;
import org.objectweb.petals.binding.exception.SMTPBCException;
import org.objectweb.petals.component.common.handleMessagesExchange.HandleResponses;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.StringHelper;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;

public class SMTPBCJBIListener implements Runnable {

    protected SenderMail senderMail;

    private DeliveryChannel channel;

    private ComponentContext context;

    private Logger logger;

    private boolean running;

    // This is the type of message to be send simple, signed or signedAndCripted
    private String typeMail;

    public SMTPBCJBIListener(DeliveryChannel channel, Logger logger,
        ComponentContext context, Properties props, CertificatePKCS12 cert) {
        this.channel = channel;
        this.logger = logger;
        this.context = context;
        if (props != null) {
            this.senderMail = new SenderMail(props.getProperty("host"), props
                .getProperty("from"), context.getInstallRoot() + File.separator
                + "certificate" + File.separator, cert, this.logger);
            this.typeMail = props.getProperty("message.type");
        } else {
            this.logger.log(Level.SEVERE,
                "connexion properties haven't been initialize");
        }
    }

    /**
     * 
     * @param endpoint
     *            -ServiceEndpoint
     * @return email to contact External service
     * @throws SMTPBCException
     */
    public String getReceiver(ServiceEndpoint endpoint) throws SMTPBCException {
        String str = null;
        try {
            if (endpoint != null) {
                str = WSDLHelper
                    .getServiceAddressFromWSDLDocument(context
                        .getEndpointDescriptor(endpoint), endpoint
                        .getServiceName());
            }
        } catch (Exception e) {
            throw new SMTPBCException(e.getMessage());
        }
        return str;
    }

    public boolean handleInMessage(MessageExchange msg) throws SMTPBCException {
        // Read the IN message
        try {
            String content = null;
            boolean send = false;
            NormalizedMessage nm = msg.getMessage("IN");

            try {
                if (nm.getContent() != null) {
                    content = SourceHelper.createString(nm.getContent());
                }

            } catch (Exception e) {
                throw new SMTPBCException(e.getMessage());
            }

            if (StringHelper.isEmpty(content)) {
                logger.log(Level.INFO, "content is null");
            }
            Set attachId = nm.getAttachmentNames();
            Set<DataHandler> attachs = new HashSet<DataHandler>();
            if (attachId != null) {
                for (Object object : attachId) {
                    attachs.add(nm.getAttachment((String) object));
                }
            }

            /*
             * Search in wsdl email address of external service
             */
            String to = this.getReceiver(msg.getEndpoint());
            String subject = null;
            if (msg.getOperation() != null) {
                subject = msg.getOperation().getLocalPart();
            }
            /*
             * If recipient address exist so mail is send else Warning is raised
             */
            if (to != null) {

                this.senderMail.sendMail(this.typeMail, content, to, subject,
                    attachs);
                send = true;

            } else {

                logger.log(Level.WARNING, "receiver don't exist");
            }
            /*
             * Set Status, auxiliary function to test
             */
            msg.setStatus(ExchangeStatus.DONE);

            return send;
        } catch (Exception e) {
            try {
                msg.setStatus(ExchangeStatus.DONE);
            } catch (javax.jbi.messaging.MessagingException e2) {
                this.logger.log(Level.WARNING, "Error setting message status");
            }
            throw new SMTPBCException(e.getMessage());
        }
    }

    public void processInOnlyMessages(MessageExchange msg) {
        try {
            handleInMessage(msg);
        } catch (Exception e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }
    }

    public void processRobustInOnlyMessages(MessageExchange msg) {
        try {
            if (!handleInMessage(msg)) {
                HandleResponses.faultMessageExchange(msg, new SMTPBCException(
                    "Recipient address unknonw"), "unknow");
            }
        } catch (Exception e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }

    }

    public void run() {

        running = true;
        MessageExchange messageExchange = null;
        while (running) {
            try {
                messageExchange = channel.accept();
                if (messageExchange != null) {
                    if (!(messageExchange.getStatus()
                        .equals(ExchangeStatus.DONE))) {
                        {
                            if ((messageExchange instanceof InOnly)
                                && !(messageExchange instanceof RobustInOnly)) {
                                // Handle InOnly message
                                processInOnlyMessages(messageExchange);
                            } else if ((messageExchange instanceof InOptionalOut)
                                || (messageExchange instanceof RobustInOnly)) {
                                processRobustInOnlyMessages(messageExchange);
                            } else if (messageExchange instanceof InOut) {
                                // Handle InOut message
                                try {
                                    HandleResponses.faultMessageExchange(
                                        messageExchange, new SMTPBCException(
                                            "InOut exchanges don't support"),
                                        "505");
                                } catch (JBIException e) {
                                    this.logger.log(Level.WARNING, e
                                        .getMessage());
                                }
                                messageExchange.setStatus(ExchangeStatus.DONE);
                            } else if (messageExchange.getStatus().equals(
                                ExchangeStatus.ERROR)) {
                                try {
                                    this.logger.log(Level.WARNING, SourceHelper
                                        .createStringContent(messageExchange
                                            .getFault().getContent()));

                                } catch (Exception e) {
                                    this.logger.log(Level.WARNING,
                                        "Error getting error message");
                                }
                            }
                            channel.send(messageExchange);
                        }
                    } else {
                        this.logger.log(Level.INFO, "Receive state DONE");
                    }
                }
            } catch (MessagingException e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
            }
        }
    }

    public void stopProcessing() {
        this.running = false;
    }

}
