/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.smtpbc.listeners;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;
import static org.easymock.classextension.EasyMock.createMock;

import java.io.File;
import java.io.FileInputStream;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.mail.Message;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ebmwebsourcing.b2bgw.security.SecurityManager;
import org.ebmwebsourcing.b2bgw.security.beans.CertificatePKCS12;
import org.objectweb.petals.binding.exception.SMTPBCException;
import org.objectweb.petals.binding.smtpbc.listeners.SMTPBCJBIListener;
import org.objectweb.petals.binding.smtpbc.listeners.SenderMail;
import org.objectweb.petals.binding.smtpbc.mock.MockMessageExchange;
import org.objectweb.petals.binding.smtpbc.mock.MockNormalizedMessage;
import org.objectweb.petals.binding.smtpbc.mock.MockSenderMail;
import org.objectweb.petals.binding.smtpbc.mock.MockServiceEndpoint;
import org.objectweb.petals.component.common.handleMessagesExchange.HandleResponses;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.w3c.dom.Document;

/**
 * test SMTPBCJBIListener class, and its collaborator classes. MessageMime and
 * SenderMail classes
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class SMTPBCJBIListenerTest extends TestCase {

    private String baseDir;

    private CertificatePKCS12 cert;

    private Properties props;

    DeliveryChannel channel;

    SMTPBCJBIListener jbiListener;

    private Logger logger;

    public void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);

        /*
         * Create Mock Objects
         */
        channel = createMock(DeliveryChannel.class);

        props = new Properties();
        props.load(new FileInputStream(baseDir
            + "src" + File.separator + "test" + File.separator + "files"
            + File.separator + "META-INF" + File.separator
            + "connection.properties"));
        cert = SecurityManager.loadPKCS12Certificate(new FileInputStream(
            baseDir
                + "src" + File.separator + "conf" + File.separator
                + "smtpbc.p12"), "password");
        logger = EasyMock.createMock(Logger.class);
    }

    public void testGetReceiver() {

        try {

            MockServiceEndpoint ep = new MockServiceEndpoint();
            ep.setEndpointName("activeService");

            String str = baseDir
                + "src" + File.separator + "test" + File.separator + "files";

            Document wsdl = WSDLHelper.createDocumentFromWSDL(new File(baseDir
                + "src" + File.separator + "test" + File.separator + "files"
                + File.separator + "falseSmtpbc.wsdl"));
            ComponentContext contextFictive = createMock(ComponentContext.class);

            expect(contextFictive.getEndpointDescriptor(null)).andReturn(null);
            expect(contextFictive.getEndpointDescriptor(ep)).andReturn(wsdl);
            expect(contextFictive.getInstallRoot()).andReturn(str);

            replay(contextFictive);

            SMTPBCJBIListener jbilistener = new SMTPBCJBIListener(null, Logger
                .getAnonymousLogger(), contextFictive, props, null);

            // if endpoint exist
            String receiver = jbilistener.getReceiver(ep);
            assert (receiver == null);

            // if enpoint don't exist
            receiver = jbilistener.getReceiver(null);
            assert (receiver != null);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testHandleInMessage() {
        String str = baseDir
            + "src" + File.separator + "test" + File.separator + "files";

        ComponentContext contextFictive = createMock(ComponentContext.class);

        expect(contextFictive.getInstallRoot()).andReturn(str).anyTimes();

        replay(contextFictive);

        jbiListener = new SMTPBCJBIListener(null, EasyMock
            .createMock(Logger.class), contextFictive, props, null) {
            public String getReceiver(ServiceEndpoint endpoint) {
                if (endpoint != null) {
                    if (endpoint.getEndpointName().equals("activeEndpoint")) {
                        return "to@receiver";
                    }
                }
                return null;
            }

        };
        MockSenderMail sendermail = new MockSenderMail(null, null, null, null,
            this.logger);

        jbiListener.senderMail = sendermail;

        MockMessageExchange exchange = new MockMessageExchange();
        ServiceEndpoint ep = createMock(ServiceEndpoint.class);
        expect(ep.getEndpointName()).andReturn("activeEndpoint");
        replay(ep);

        exchange.setEndpoint(ep);

        MockNormalizedMessage nm = new MockNormalizedMessage();
        
        Set<String> ids = new HashSet<String>();
        ids.add("id1");

        File file = new File(baseDir
            + "src" + File.separator + "test" + File.separator + "files"
            + File.separator + "attach.txt");
        Set<DataHandler> attach = new HashSet<DataHandler>();

        attach.add(new DataHandler(new FileDataSource(file)));

        try {
            exchange.setMessage(nm, "IN");
            HandleResponses.buildInMessage(exchange, "<test/>", QName.valueOf("service"),
                "operation", attach);
            /*
             * test 1: Endpoint is active, so recipient contact is not null
             */
            boolean ret = jbiListener.handleInMessage(exchange);
            assertEquals(ret, true);
            assertTrue(sendermail.isSend());

            /*
             * test 1: Endpoint is inactive, so recipient contact is null
             */

            exchange.setEndpoint(null);
            ret = jbiListener.handleInMessage(exchange);
            assertEquals(ret, false);
            verify(ep);

        } catch (MessagingException e1) {

        } catch (SMTPBCException e) {
            e.printStackTrace();
        } catch (JBIException e) {
            e.printStackTrace();
        }

    }

    public void testProcessInOnlyMessages() {
        ComponentContext contextFictive = createMock(ComponentContext.class);
        String str = baseDir
            + File.separator + "src" + File.separator + "test" + File.separator
            + "files";
        expect(contextFictive.getInstallRoot()).andReturn(str);
        replay(contextFictive);
        jbiListener = new SMTPBCJBIListener(null, EasyMock
            .createMock(Logger.class), contextFictive, props, null) {
            public boolean handleInMessage(MessageExchange msg) {
                if (msg != null)
                    return true;
                else
                    return false;
            }
        };

        MockMessageExchange exchange = null;
        exchange = new MockMessageExchange();

        /*
         * MessageExchange has a content
         */
        try {
            jbiListener.processInOnlyMessages(exchange);
        } catch (Exception e) {
            e.printStackTrace();
        }
        /*
         * Message exchange is a fault
         */
        try {
            jbiListener.processInOnlyMessages(null);
        } catch (Exception e) {
            e.printStackTrace();
        }
        verify(contextFictive);
    }

    public void testProcessRobustInOnlyMessages() {
        ComponentContext contextFictive = createMock(ComponentContext.class);

        String str = baseDir
            + File.separator + "src" + File.separator + "test" + File.separator
            + "files";
        expect(contextFictive.getInstallRoot()).andReturn(str);

        jbiListener = new SMTPBCJBIListener(null, EasyMock
            .createMock(Logger.class), contextFictive, null, null) {
            public boolean handleInMessage(MessageExchange msg) {
                if (msg.getError() == null)
                    return true;
                else {

                    return false;
                }
            }
        };

        MockMessageExchange exchange = null;
        exchange = new MockMessageExchange();

        /*
         * MessageExchange has a content
         */
        try {
            jbiListener.processRobustInOnlyMessages(exchange);
        } catch (Exception e) {
            e.printStackTrace();
        }
        /*
         * Message exchange is a fault
         */

        try {
            exchange.setError(new Exception());
            jbiListener.processRobustInOnlyMessages(exchange);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void testSendmail() {

        try {

            SenderMail senderMail = new SenderMail("192.168.5.2",
                "testsmtp@ebmwebsourcing.org", baseDir
                    + "src" + File.separator + "conf" + File.separator, cert,
                logger) {

                public void transport(Message msg) {
                }
            };

            /**
             * Create a new MessageMime class
             */

            File file = new File(baseDir
                + "src" + File.separator + "test" + File.separator + "files"
                + File.separator + "attach.txt");
            Set<DataHandler> attach = new HashSet<DataHandler>();

            attach.add(new DataHandler(new FileDataSource(file)));
            /**
             * Test 1: Case Right parameters
             */
            senderMail.sendMail("simple", "test",
                "testsmtp@ebmwebsourcing.org", "service", attach);

            /**
             * Test 2: case with invalidate message type
             */
            try {

                senderMail.sendMail("otherType", "test",
                    "testsmtp@ebmwebsourcing.org", "service", attach);
                fail("\n*** Exception about type message ***\n");
            } catch (Exception e) {
            }
            /**
             * Test 3: Subject is null, so operation of external service is null
             */
            senderMail.sendMail("simple", "test",
                "testsmtp@ebmwebsourcing.org", null, null);

            /**
             * Test 4: Body is null.
             */
            senderMail.sendMail("simple", null, "testsmtp@ebmwebsourcing.org",
                "service", null);

            /**
             * Test 5 sender address is invalidate
             */
            try {
                senderMail.sendMail("simple", "test", "", "service", null);
                fail("*** Exception notifying invalidate recipient address ***");
            } catch (Exception e) {
            }
            /**
             * send sign message
             */
            senderMail.sendMail("signed", "test",
                "testsmtp@ebmwebsourcing.org", "service", null);
            //

            /**
             * send signed and encrypted message
             */
            senderMail.sendMail("signedAndCrypted", "test",
                "testsmtp@ebmwebsourcing.org", "service", null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error sending mail");
        }

    }
}
