/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: EvalRate.java 10:42:15 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.mortgage.evalrate;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.logging.ComponentLogger;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * This service engine is used in the Mortgage demonstration to evaluate the
 * mortgage rate
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class EvalRate implements Component, ComponentLifeCycle {

    private static final String ENDPOINT = "EvalRateEndpoint";

    private static final String SERVICE = "EvalRateService";

    private DeliveryChannel channel;

    private ComponentContext context;

    private ServiceEndpoint endpointReference;

    private EvalRateListener listener;

    private Logger logger;

    public ObjectName getExtensionMBeanName() {
        return null;
    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    /**
     * If arg0.service = HelloworldService, return the wsdl description
     */
    public Document getServiceDescription(ServiceEndpoint arg0) {
        if (new QName(SERVICE).equals(arg0.getServiceName())) {
            File wsdlFile = new File(context.getInstallRoot() + "/META-INF/"
                + SERVICE + ".wsdl");
            return WSDLHelper.createDocumentFromWSDL(wsdlFile);
        }
        return null;
    }

    public ServiceUnitManager getServiceUnitManager() {
        return null;
    }

    public void init(ComponentContext context) throws JBIException {
        Logger log = context.getLogger("", null);
        logger = new ComponentLogger(log, log.getName(), log
            .getResourceBundleName(), "MortgageEvalRate");
        logger.log(Level.INFO, "");
        this.context = context;
        this.channel = context.getDeliveryChannel();
    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        logger.log(Level.INFO, "accept the exchange");
        return true;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return true;
    }

    /**
     * If requested service is EvalRateService, return the endpoint
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;
        if (arg0 != null) {
            Node service = XMLHelper.findChild(arg0, "service", true);
            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                    "name");
                if (SERVICE.equals(serviceRequested)) {
                    result = endpointReference;
                }
            }
        }
        return result;
    }

    public void shutDown() throws JBIException {
        logger.log(Level.INFO, "");
        this.listener.stopProcessing();
    }

    public void start() throws JBIException {
        logger.log(Level.INFO, "");
        try {
            this.listener = new EvalRateListener(this.channel, logger);
            Thread listenerThread = new Thread(this.listener);
            listenerThread.start();
            endpointReference = this.context.activateEndpoint(
                new QName(SERVICE), ENDPOINT);
        } catch (MessagingException e) {
            throw new JBIException(e);
        }

    }

    public void stop() throws JBIException {
        logger.log(Level.INFO, "");
        this.listener.stopProcessing();
    }

}
