package org.objectweb.petals.demo.mortgage.workflow;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.listener.IMessageExchangeProcessor;
import org.objectweb.petals.component.common.listener.MessageExchangeListener;
import org.objectweb.petals.component.common.logging.ComponentLogger;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

public class MortgageWorkflow implements Component, ComponentLifeCycle {

    private ComponentContext context;

    private MessageExchangeListener listener;

    private DeliveryChannel channel;

    private Logger logger;

    private ServiceEndpoint endpointReference;

    private static final String SERVICE = "MortgageWorkflow";

    private static final String SERVICE_NS = "http://petals.objectweb.org/";

    public static final String ENDPOINT1 = "simpleMortgageProfiler";

    public static final String ENDPOINT2 = "detailedMortgageProfiler";

    public void init(ComponentContext context) throws JBIException {
        Logger log = context.getLogger("", null);
        logger = new ComponentLogger(log, log.getName(), log
            .getResourceBundleName(), "MortgageWorkflow");
        logger.log(Level.INFO, "");

        this.context = context;
        this.channel = context.getDeliveryChannel();
    }

    public void shutDown() throws JBIException {
        logger.log(Level.INFO, "");
        this.listener.terminate();

    }

    public void start() throws JBIException {
        logger.log(Level.INFO, "");

        try {
            IMessageExchangeProcessor processor = new MortgageWorkflowProcessor(
                    context, channel, logger);
            this.listener = new MessageExchangeListener(
                    this.channel,
                    processor,
                    MessageExchangeListener.IgnoredStatus.DONE_AND_ERROR_IGNORED);

            this.listener.listen();

            endpointReference = this.context.activateEndpoint(new QName(
                    SERVICE_NS, SERVICE), ENDPOINT1);
            this.context.activateEndpoint(new QName(SERVICE_NS, SERVICE),
                    ENDPOINT2);
        } catch (MessagingException e) {
            throw new JBIException(e);
        }

    }

    public void stop() throws JBIException {
        logger.log(Level.INFO, "");
        this.listener.terminate();

    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    /**
     * If arg0.service = Mortgage service, return the wsdl description
     */
    public Document getServiceDescription(ServiceEndpoint arg0) {
        if (new QName(SERVICE_NS, SERVICE).equals(arg0.getServiceName())) {
            File wsdlFile = new File(context.getInstallRoot() + File.separator
                    + "META-INF" + File.separator + "xlinker"+ File.separator + "MortgageProfiling.wsdl");

            return WSDLHelper.createDocumentFromWSDL(wsdlFile);
        }
        return null;
    }

    public ServiceUnitManager getServiceUnitManager() {
        return null;
    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
            MessageExchange arg1) {

        logger.log(Level.INFO, "accept the exchange");
        return true;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
            MessageExchange arg1) {
        return true;
    }

    /**
     * If requested service is MortgageWorkflow, return the endpoint
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;

        if (arg0 != null) {
            Node service = XMLHelper.findChild(arg0, "service", true);

            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                        "name");

                if (SERVICE.equals(serviceRequested)) {
                    result = endpointReference;
                }
            }
        }
        return result;
    }

    public ObjectName getExtensionMBeanName() {
        // TODO Auto-generated method stub
        return null;
    }

}
