/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentServiceImpl.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jbi.management.AdminServiceMBean;
import javax.jbi.management.LifeCycleMBean;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.jbi.management.service.util.XMLResult.CauseFramework;
import org.objectweb.petals.jbi.management.service.util.XMLResult.MessageType;
import org.objectweb.petals.jbi.management.service.util.XMLResult.TaskResult;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ParameterCheckHelper;
import org.objectweb.util.monolog.api.Logger;

import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_ADMIN_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;

/**
 * JBI deployment service implementation. TODO map recovery service with fractal
 * 
 * @author Olivier Fabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
public class DeploymentServiceImpl implements DeploymentService,
        BindingController, LifeCycleController {

    protected static final String[] CLIENT_INTERFACES = new String[] {
        PLATFORM_REPOSITORY_NAME, JBI_LIFECYCLEMANAGER_NAME, JBI_ADMIN_NAME,
        JBI_SYSTEMSTATE_NAME, JBI_ENDPOINTSERVICE_NAME};

    /**
     * JBI Container :: Admin Service
     */
    protected AdminServiceMBean adminSrv;

    /**
     * List of already deployed service assembly.
     */
    protected List<String> deployedSACache;

    protected EndpointService endpointService;

    /**
     * Component lifecycle state.
     */
    protected String fcState;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog Logger instance.
     */
    protected Logger logger;

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected LifeCycleManagerService managerService;

    /**
     * Package Handler
     */
    protected PackageHandler packageHandler;

    /**
     * JBI Container :: Recovery Service
     */
    protected SystemState recoverySrv;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * Default constructor.
     */
    public DeploymentServiceImpl() {
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = (Logger) serverItf;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            repositorySrv = (RepositoryService) serverItf;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            managerService = (LifeCycleManagerService) serverItf;
        } else if (JBI_ADMIN_NAME.equals(clientItfName)) {
            adminSrv = (AdminServiceMBean) serverItf;
        } else if (JBI_SYSTEMSTATE_NAME.equals(clientItfName)) {
            recoverySrv = (SystemState) serverItf;
        } else if (JBI_ENDPOINTSERVICE_NAME.equals(clientItfName)) {
            endpointService = (EndpointService) serverItf;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#canDeployToComponent(java.lang.String)
     */
    public boolean canDeployToComponent(String componentName) {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
                "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
                "componentName", log);
        boolean result = false;
        ComponentLifeCycle clc = managerService
                .getComponentByName(componentName);
        result = clc != null
                && LifeCycleAbstract.STARTED.equals(clc.getCurrentState())
                && clc.getComponent().getServiceUnitManager() != null;
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#deploy(java.lang.String)
     */
    public String deploy(String serviceAssemblyZipUrl) throws Exception {
        log.start();
        String result = null;

        try {
            /*
             * 0. Checking parameters
             */
            ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyZipUrl,
                    "serviceAssemblyZipUrl", log);
            ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyZipUrl,
                    "serviceAssemblyZipUrl", log);

            /*
             * 1. Transforming URL into URI
             */
            URI archiveURI = packageHandler.processAndGetPackageURI(
                    serviceAssemblyZipUrl, true);

            /*
             * 2. Loading descriptor
             */
            JBIDescriptor descriptor = packageHandler
                    .loadDescriptor(archiveURI);

            /*
             * 3. Performing service assembly deployment
             */
            result = processSADeployment(archiveURI, descriptor, false);
        } catch (Throwable e) {
            // Catch all throwables to prevent exotic exception type (not easily
            // deserializable) throwing
            log.error(e.getMessage(), e);
            throw new Exception(e.getMessage());
        }

        log.info(result);
        log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getComponentsForDeployedServiceAssembly(java.lang.String)
     */
    public String[] getComponentsForDeployedServiceAssembly(
            String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        List<String> result = new ArrayList<String>();
        ServiceAssemblyLifeCycle saLifeCycle = managerService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            List<ServiceUnit> sus = saLifeCycle.getServiceAssembly()
                    .getServiceUnits();
            for (ServiceUnit unit : sus) {
                result.add(unit.getTargetComponentName());
            }
        } else {
            String msg = "Cannot retrieve service assembly with the specified name : "
                    + serviceAssemblyName;
            log.error(msg);
            throw new Exception(msg);
        }
        return result.toArray(new String[0]);
    }

    public List<String> getDeployedSACache() {
        return deployedSACache;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssemblies()
     */
    public String[] getDeployedServiceAssemblies() throws Exception {
        Set<String> sas = managerService.getServiceAssemblyLifeCycles()
                .keySet();
        return sas.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssembliesForComponent(java.lang.String)
     */
    public String[] getDeployedServiceAssembliesForComponent(
            String componentName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
                "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
                "componentName", log);
        Set<String> result = new HashSet<String>();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
                .getServiceAssemblyLifeCycles();
        for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTargetComponentName())) {
                    result.add(salc.getServiceAssembly().getIdentification()
                            .getName());
                }
            }
        }
        return result.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceUnitList(java.lang.String)
     */
    public String[] getDeployedServiceUnitList(String componentName)
        throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
                "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
                "componentName", log);
        Set<String> result = new HashSet<String>();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
                .getServiceAssemblyLifeCycles();
        for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTargetComponentName())) {
                    result.add(unit.getIdentification().getName());
                }
            }
        }
        return result.toArray(new String[0]);
    }

    /*
     * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
     */
    public String getFcState() {
        return org.objectweb.fractal.api.control.LifeCycleController.STARTED
                .equals(fcState) ? org.objectweb.fractal.api.control.LifeCycleController.STARTED
            : org.objectweb.fractal.api.control.LifeCycleController.STOPPED;
    }

    public String getServiceAssemblyDescriptor(String serviceAssemblyName)
        throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        String result = "Service Assembly doesn't exist :"
                + serviceAssemblyName;
        ServiceAssemblyLifeCycle saLifeCycle = managerService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            URI installationRoot = saLifeCycle.getInstallationRoot();
            // Load JBI descriptor as string
            URI jbiXmlUri = installationRoot
                    .resolve(JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE
                            .replace('\\', '/'));
            result = getJbiDescriptorAsString(jbiXmlUri);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getState(java.lang.String)
     */
    public String getState(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        String result = null;
        String msg;
        ServiceAssemblyLifeCycle saLifeCycle = managerService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            result = saLifeCycle.getCurrentState();
        } else {
            msg = "Cannot retrieve service "
                    + "assembly with the specified name : "
                    + serviceAssemblyName;
            log.error(msg);
            throw new Exception(msg);
        }
        if (!(LifeCycleMBean.SHUTDOWN.equals(result)
                || LifeCycleMBean.STARTED.equals(result) || LifeCycleMBean.STOPPED
                .equals(result))) {
            msg = "Service Assembly state isn't correctly defined";
            log.error(msg);
            throw new Exception(msg);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#isDeployedServiceUnit(java.lang.String,
     *      java.lang.String)
     */
    public boolean isDeployedServiceUnit(String componentName,
            String serviceUnitName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceUnitName,
                "serviceUnitName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceUnitName,
                "serviceUnitName", log);
        ParameterCheckHelper.isNullParameterWithLog(componentName,
                "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
                "componentName", log);
        boolean result = false;
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
                .getServiceAssemblyLifeCycles();
        for (Iterator iter = saLifeCycles.values().iterator(); iter.hasNext()
                && !result;) {
            ServiceAssemblyLifeCycle salc = (ServiceAssemblyLifeCycle) iter
                    .next();
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
            for (Iterator iter2 = sus.iterator(); iter2.hasNext() && !result;) {
                ServiceUnit unit = (ServiceUnit) iter2.next();
                if (componentName.equals(unit.getTargetComponentName())
                        && serviceUnitName.equals(unit.getIdentification()
                                .getName())) {
                    result = true;
                }
            }
        }
        return result;
    }

    /*
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /*
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {
        if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            return repositorySrv;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            return managerService;
        } else if (JBI_ADMIN_NAME.equals(clientItfName)) {
            return adminSrv;
        } else if (JBI_SYSTEMSTATE_NAME.equals(clientItfName)) {
            return recoverySrv;
        } else if (JBI_ENDPOINTSERVICE_NAME.equals(clientItfName)) {
            return endpointService;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.jbi.management.service.DeploymentService#processSADeployment(java.net.URI,
     *      org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor)
     */
    public String processSADeployment(URI saArchiveURI, JBIDescriptor descriptor)
        throws Exception {
        return processSADeployment(saArchiveURI, descriptor, true);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#shutDown(java.lang.String)
     */
    public String shutDown(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        log.start();

        String msg = null;
        XMLResult xmlResult = new XMLResult("shutdown");

        ServiceAssemblyLifeCycle saLifeCycle = managerService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            try {
                saLifeCycle.shutDown();
            } catch (Exception e) {
                log.error(e);
                msg = e.getMessage();
                throw new Exception(msg);
            }
        } else {
            msg = "The service assembly \"" + serviceAssemblyName
                    + "\" is not deployed";
            throw new Exception(msg);
        }
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                "Successfully shutdown service assembly : {1}",
                new String[] {serviceAssemblyName}, CauseFramework.YES);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                    .getClassLoader().getResourceAsStream(
                            "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        log.info(output);
        log.end();
        return output;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
     */
    public String start(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        log.start();

        String msg = null;
        XMLResult xmlResult = new XMLResult("start");

        ServiceAssemblyLifeCycle saLifeCycle = managerService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            try {
                saLifeCycle.start();
            } catch (Exception e) {
                e.printStackTrace();
                msg = e.getMessage();
                throw new Exception(msg);
            }
        } else {
            msg = "The service assembly \"" + serviceAssemblyName
                    + "\" is not deployed";
            throw new Exception(msg);
        }
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                "Successfully started service assembly : {1}",
                new String[] {serviceAssemblyName}, CauseFramework.YES);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                    .getClassLoader().getResourceAsStream(
                            "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        log.info(output);
        log.end();
        return output;
    }

    /*
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {
        init();
        fcState = org.objectweb.fractal.api.control.LifeCycleController.STARTED;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#stop(java.lang.String)
     */
    public String stop(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", log);
        log.start();

        String msg = null;
        XMLResult xmlResult = new XMLResult("stop");

        ServiceAssemblyLifeCycle saLifeCycle = managerService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            try {
                saLifeCycle.stop();
            } catch (Exception e) {
                log.error(e);
                msg = e.getMessage();
                throw new Exception(msg);
            }
        } else {
            msg = "The service assembly \"" + serviceAssemblyName
                    + "\" is not deployed";
            throw new Exception(msg);
        }
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                "Successfully stopped service assembly : {1}",
                new String[] {serviceAssemblyName}, CauseFramework.YES);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                    .getClassLoader().getResourceAsStream(
                            "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        log.info(output);
        log.end();
        return output;
    }

    /*
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {
        stop();
        fcState = org.objectweb.fractal.api.control.LifeCycleController.STOPPED;
    }

    /*
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = null;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            repositorySrv = null;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            managerService = null;
        } else if (JBI_ADMIN_NAME.equals(clientItfName)) {
            adminSrv = null;
        } else if (JBI_SYSTEMSTATE_NAME.equals(clientItfName)) {
            recoverySrv = null;
        } else if (JBI_ENDPOINTSERVICE_NAME.equals(clientItfName)) {
            endpointService = null;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#undeploy(java.lang.String)
     */
    public String undeploy(String serviceAssemblyName) throws Exception {
        log.start();

        String output = "";
        try {
            ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
                    "serviceAssemblyZipUrl", log);
            ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
                    "serviceAssemblyZipUrl", log);

            /*
             * Execute the undeployment process of a service assembly
             */
            SADeploymentManager manager = new SADeploymentManager(null, null,
                    packageHandler, endpointService, this, repositorySrv,
                    recoverySrv, log, managerService);
            output = manager.undeploySA(serviceAssemblyName);

        } catch (Throwable e) {
            // Catch all throwables to prevent exotic exception type (not easily
            // deserializable) throwing
            log.error(e.getMessage(), e);
            throw new Exception(e.getMessage());
        }

        log.info(output);
        log.end();
        return output;
    }

    /**
     * Return a string representation of the JBI descriptor. This descriptor in
     * found thanks to the jbiXmlUri
     * 
     * @param jbiXmlUri
     *            the URI of the deployment descriptor
     * @return a string representing the JBI deployment descriptor
     */
    protected String getJbiDescriptorAsString(URI jbiXmlUri) {
        JBIDescriptorBuilder builder = new JBIDescriptorBuilder(jbiXmlUri,
                (java.util.logging.Logger) logger);
        return builder.getJbiDescriptorAsString();
    }

    protected void init() {
        log = new LoggingUtil(logger);
        log.start();

        /*
         * initialize packages loading cache
         */
        deployedSACache = new ArrayList<String>();

        /*
         * Initialize package handler
         */
        packageHandler = new PackageHandler(logger, repositorySrv);

        log.end();
    }

    /**
     * Deploy a new Service Assembly to the Petals platform. Deployment steps :
     * create service unit and service assembly life cycles, deploy sus to
     * components, register lifecycles to the LifeCycleManagerService, register
     * the sa to the loaded sa map and su to the loaded su map
     * 
     * @param saArchiveURI
     *            URI representing the sa zip archive, must be non-null
     * @param descriptor
     *            the sa <code>JBIDescriptor</code> must be non-null
     * @param recovery
     *            used to know if this method is called by the recovery service
     *            or not. True, called by the recovery service, false otherwise.
     * @return the XML String representation of the task result (see section 6.9
     *         of the JBI Spec), must be non null, non empty and comform to JBI
     *         management result/status XML schema
     * @throws Exception
     *             if deployment process fails
     */
    protected String processSADeployment(URI saArchiveURI,
            JBIDescriptor descriptor, boolean recovery) throws Exception {
        String result = null;

        /*
         * Retrieve assembly production element into JBI descriptor
         */
        ServiceAssembly assembly = descriptor.getServiceAssembly();
        if (assembly == null) {
            String msg = "Given archive isn't a service assembly archive : "
                    + "missing service assembly production element into jbi descriptor";
            log.error(msg);
            throw new ManagementException(msg);
        }

        /*
         * Execute the deployment process of a service assembly
         */
        SADeploymentManager manager = new SADeploymentManager(assembly,
                saArchiveURI, packageHandler, endpointService, this,
                repositorySrv, recoverySrv, log, managerService);
        result = manager.deploySA(recovery);

        return result;
    }

    protected void stop() {
        log.start();

        logger = null;

        deployedSACache = null;

        log.end();
    }

    /**
     * Shutdown the DeploymentService. All service assemblies are undeployed.
     */
    public void shutdown() throws Exception {
        // Undeploy all service assemblies
        List<String> removeList = new ArrayList<String>(deployedSACache);
        for (String saName : removeList) {
            undeploy(saName);
        }
    }

}
