/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsAdminService.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.admin.service;

import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_DEPLOYMENT_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_INSTALLATION_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JORAM_AGENT_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_TRANSPORTER_NAME;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.util.monolog.api.Logger;

import org.objectweb.petals.jbi.management.service.DeploymentService;
import org.objectweb.petals.jbi.management.service.InstallationService;
import org.objectweb.petals.jbi.transport.Transporter;
import org.objectweb.petals.kernel.joram.JoramAgent;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This class is used for Petals global Administration
 * 
 * @author alouis - eBMWebsourcing
 */
public class PetalsAdminService implements PetalsAdminServiceMBean,
    BindingController, LifeCycleController {

    private static final String[] CLIENT_INTERFACES = new String[] {
        JBI_DEPLOYMENT_NAME, JBI_INSTALLATION_NAME,
        JORAM_AGENT_NAME, JBI_TRANSPORTER_NAME};

    /**
     * Logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog
     */
    protected Logger logger;

    /**
     * JBI :: JMX Deployment
     */
    protected DeploymentService deployment;

    /**
     * JBI :: JMX Installation
     */
    protected InstallationService installation;

    protected JoramAgent joram;

    protected Transporter transporter;

    /**
     * Component lifecycle state.
     */
    private String fcState;

    /**
     * uninstalled serviceassemblies, components and sharelibraries endpoints
     * are removed the container is removed from the global configuration
     * 
     * @throws Exception
     */
    public void shutdownContainer() throws Exception{
        log.start();
        log.info("Shutdown the server. It will be removed from the Petals network.");
        try {
            deployment.shutdown();
        } catch (Exception e) {
            log.error("Can not shutdown the DeploymentService",e);
        } 
        try {
            installation.shutdown();
        } catch (Exception e) {
            log.error("Can not shutdown the InstallationService",e);
        }
        try {
            transporter.shutdown();
        } catch (Exception e) {
            log.error("Can not shutdown the Transporter",e);
        }
        try {
            joram.shutdown();
        } catch (Exception e) {
            log.error("Can not shutdown Joram Agent",e);
        }
        log.end();
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
     */
    public String getFcState() {
        return STARTED.equals(fcState) ? STARTED : STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {

       if (JBI_DEPLOYMENT_NAME
            .equals(clientItfName)) {
            return deployment;
        } else if (JBI_INSTALLATION_NAME
            .equals(clientItfName)) {
            return installation;
        } else if (JORAM_AGENT_NAME.equals(clientItfName)) {
            return joram;
        } else if (JBI_TRANSPORTER_NAME
            .equals(clientItfName)) {
            return transporter;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {

        log = new LoggingUtil(logger);

        fcState = STARTED;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {

        fcState = STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {

        if (LOGGER_NAME.equals(clientItfName)) {
            logger = (Logger) serverItf;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (JBI_DEPLOYMENT_NAME
            .equals(clientItfName)) {
            deployment = (DeploymentService) serverItf;
        } else if (JBI_INSTALLATION_NAME
            .equals(clientItfName)) {
            installation = (InstallationService) serverItf;
        } else if (JORAM_AGENT_NAME.equals(clientItfName)) {
            joram = (JoramAgent) serverItf;
        } else if (JBI_TRANSPORTER_NAME.equals(clientItfName)) {
            transporter = (Transporter) serverItf;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {

        if (LOGGER_NAME.equals(clientItfName)) {
            logger = null;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (JBI_DEPLOYMENT_NAME
            .equals(clientItfName)) {
            deployment = null;
        } else if (JBI_INSTALLATION_NAME
            .equals(clientItfName)) {
            deployment = null;
        } else if (JORAM_AGENT_NAME.equals(clientItfName)) {
            joram = null;
        } else if (JBI_TRANSPORTER_NAME.equals(clientItfName)) {
            transporter = null;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

}
