/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SUDeploymentManagerTest.java 154 18 aot 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.lang.reflect.Method;
import java.net.URI;
import java.net.URISyntaxException;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;

import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.ServiceUnitLifeCycle;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.LoggingUtil;

import static org.easymock.EasyMock.*;
import static org.easymock.classextension.EasyMock.*;

import junit.framework.TestCase;

/**
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SUDeploymentManagerTest extends TestCase {

    @Override
    protected void setUp() {

    }

    public void testCreateNewServiceUnitLifeCycle() throws ManagementException {
        /*
         * Test Case 1
         */
        /*
         * Create Test parameters
         */
        ServiceUnitLifeCycle result = null;
        ServiceUnit suDesc = createMock(ServiceUnit.class);
        String rootPath = "rootPath";
        LoggingUtil log = createMock(LoggingUtil.class);
        LifeCycleManagerService serviceManager = createMock(LifeCycleManagerImpl.class);
        Identification id = createMock(Identification.class);
        String suName = "ServiceUnitName";
        SUDeploymentManager deploymentManager = new SUDeploymentManager(suDesc,
                null, null, null, serviceManager, log, null, null);
        /*
         * Initiliaze Mocks
         */
        expect(id.getName()).andReturn(suName);
        expect(suDesc.getIdentification()).andReturn(id);
        replay(id);
        replay(suDesc);
        /*
         * Run Test Case 1
         */
        result = deploymentManager.createNewServiceUnitLifeCycle(rootPath);
        assertNotNull(
                "Result of invokation should be a new instance of ServiceUnitLifeCycle",
                result);
        verify(id);
        verify(suDesc);
    }

    public void testDeploySUtoComponent() {

        /*
         * Create Mocks and init parameters
         */
        Method method1 = null;
        try {
            method1 = SUDeploymentManager.class.getDeclaredMethod(
                    "getServiceUnitName", new Class[] {});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};

        SUDeploymentManager deploymentManager = createMock(
                SUDeploymentManager.class, mockedMethods);
        ServiceUnit suDesc = createMock(ServiceUnit.class);
        LifeCycleManagerService managerService = createMock(LifeCycleManagerService.class);
        ComponentLifeCycle componentLifeCycle = createMock(ComponentLifeCycle.class);
        Component component = createMock(Component.class);
        ServiceUnitManager serviceUnitManager = createMock(ServiceUnitManager.class);
        LoggingUtil log = createMock(LoggingUtil.class);
        String componentName = "bar";
        String serviceUnitName = "fooBar";
        String xmlResult = "An xml document";
        URI installationRoot = null;
        try {
            installationRoot = new URI("file:/foo");
        } catch (URISyntaxException e) {
            fail("Can't create test uri");
        }

        /*
         * Test Case 1 : component life cycle is null
         */
        // Initiliaze Mocks
        expect(suDesc.getTargetComponentName()).andReturn(componentName);
        expect(managerService.getComponentByName(componentName))
                .andReturn(null);
        replay(suDesc);
        replay(managerService);
        deploymentManager.serviceUnit = suDesc;
        deploymentManager.managerService = managerService;
        deploymentManager.log = log;
        // Run Test Case 1
        try {
            deploymentManager.deploySUtoComponent(installationRoot);
            fail("Must throw an exception if component lifecycle is null");
        } catch (ManagementException e) {
            // Do nothing, it's ok !
        }

        /*
         * Test Case 2 : service unit manager is null
         */
        reset(suDesc);
        reset(managerService);
        // Initiliaze Mocks
        expect(suDesc.getTargetComponentName()).andReturn(componentName);
        expect(managerService.getComponentByName(componentName)).andReturn(
                componentLifeCycle);
        expect(componentLifeCycle.getComponent()).andReturn(component);
        expect(component.getServiceUnitManager()).andReturn(null);
        replay(suDesc);
        replay(managerService);
        replay(component);
        replay(componentLifeCycle);
        deploymentManager.serviceUnit = suDesc;
        deploymentManager.managerService = managerService;
        deploymentManager.log = log;
        // Run Test Case 2
        try {
            deploymentManager.deploySUtoComponent(installationRoot);
            fail("Must throw an exception if sum is null");
        } catch (ManagementException e) {
            // Do nothing, it's ok !
        }

        /*
         * Test Case 3 : sum.deploy fails and throw a DeploymentException
         */
        reset(suDesc);
        reset(managerService);
        reset(component);
        reset(componentLifeCycle);
        // Initiliaze Mocks
        expect(suDesc.getTargetComponentName()).andReturn(componentName)
                .anyTimes();
        expect(managerService.getComponentByName(componentName)).andReturn(
                componentLifeCycle);
        expect(componentLifeCycle.getComponent()).andReturn(component);
        expect(component.getServiceUnitManager()).andReturn(serviceUnitManager);
        expect(deploymentManager.getServiceUnitName()).andReturn(
                serviceUnitName).anyTimes();
        try {
            expect(
                    serviceUnitManager.deploy(serviceUnitName, installationRoot
                            .getPath())).andThrow(
                    new DeploymentException("An exception"));
        } catch (DeploymentException e1) {
            e1.printStackTrace();
        }
        replay(suDesc);
        replay(managerService);
        replay(component);
        replay(componentLifeCycle);
        replay(serviceUnitManager);
        replay(deploymentManager);
        deploymentManager.serviceUnit = suDesc;
        deploymentManager.managerService = managerService;
        deploymentManager.log = log;
        // Run Test Case 3
        try {
            deploymentManager.deploySUtoComponent(installationRoot);
            fail("Must throw an exception if sum.deploy fails");
        } catch (ManagementException e) {
            // Do nothing, it's ok !
        }

        /*
         * Test Case 4 : all is ok
         */
        reset(suDesc);
        reset(managerService);
        reset(component);
        reset(componentLifeCycle);
        reset(deploymentManager);
        reset(serviceUnitManager);
        // Initiliaze Mocks
        expect(suDesc.getTargetComponentName()).andReturn(componentName)
                .anyTimes();
        expect(managerService.getComponentByName(componentName)).andReturn(
                componentLifeCycle);
        expect(componentLifeCycle.getComponent()).andReturn(component);
        expect(component.getServiceUnitManager()).andReturn(serviceUnitManager);
        expect(deploymentManager.getServiceUnitName()).andReturn(
                serviceUnitName).anyTimes();
        try {
            expect(
                    serviceUnitManager.deploy(serviceUnitName, installationRoot
                            .getPath())).andReturn(xmlResult);
        } catch (DeploymentException e1) {
            e1.printStackTrace();
        }
        replay(suDesc);
        replay(managerService);
        replay(component);
        replay(componentLifeCycle);
        replay(serviceUnitManager);
        replay(deploymentManager);
        deploymentManager.serviceUnit = suDesc;
        deploymentManager.managerService = managerService;
        deploymentManager.log = log;
        // Run Test Case 4
        try {
            String result = deploymentManager.deploySUtoComponent(installationRoot);
            assertEquals("Bad xml result returned", xmlResult, result);
        } catch (ManagementException e) {
            e.printStackTrace();
        }
    }

    public void testGetParentServiceAssemblyName() {
        /*
         * Create Mocks and init parameters
         */
        Identification identification = createMock(Identification.class);
        ServiceAssembly serviceAssembly = createMock(ServiceAssembly.class);
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        SUDeploymentManager deploymentManager = new SUDeploymentManager();
        String saName = "foo";

        // Initiliaze Mocks
        expect(identification.getName()).andReturn(saName);
        expect(serviceAssembly.getIdentification()).andReturn(identification);
        expect(serviceAssemblyLifeCycle.getServiceAssembly()).andReturn(serviceAssembly);
        replay(identification);
        replay(serviceAssembly);
        replay(serviceAssemblyLifeCycle);
        deploymentManager.saLifeCycle = serviceAssemblyLifeCycle;
        
        // Run Test Case 1
        String result = deploymentManager.getParentServiceAssemblyName();
        assertEquals("Bad sa name returned", saName, result);
    }
    
    public void testGetServiceUnitName() {
        /*
         * Create Mocks and init parameters
         */
        Identification identification = createMock(Identification.class);
        ServiceUnit serviceUnit = createMock(ServiceUnit.class);
        SUDeploymentManager deploymentManager = new SUDeploymentManager();
        String suName = "foo";

        // Initiliaze Mocks
        expect(identification.getName()).andReturn(suName);
        expect(serviceUnit.getIdentification()).andReturn(identification);
        replay(identification);
        replay(serviceUnit);
        deploymentManager.serviceUnit = serviceUnit;
        
        // Run Test Case 1
        String result = deploymentManager.getServiceUnitName();
        assertEquals("Bad sa name returned", suName, result);
    }
    
    public void testGetTargetedComponent() {
        /*
         * Create Mocks and init parameters
         */
        ServiceUnit serviceUnit = createMock(ServiceUnit.class);
        SUDeploymentManager deploymentManager = new SUDeploymentManager();
        String targetedComp = "foo";

        // Initiliaze Mocks
        expect(serviceUnit.getTargetComponentName()).andReturn(targetedComp);
        replay(serviceUnit);
        deploymentManager.serviceUnit = serviceUnit;
        
        // Run Test Case 1
        String result = deploymentManager.getTargetedComponent();
        assertEquals("Bad sa name returned", targetedComp, result);
    }
    
    public void testRegisterSULifeCycle() {
        /*
         * Create Mocks and init parameters
         */
        Method method1 = null;
        try {
            method1 = SUDeploymentManager.class.getDeclaredMethod(
                    "getServiceUnitName", new Class[] {});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};

        SUDeploymentManager deploymentManager = createMock(
                SUDeploymentManager.class, mockedMethods);
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        ServiceUnitLifeCycle serviceUnitLifeCycle = createMock(ServiceUnitLifeCycle.class);
        String suName = "foo";

        /*
         * Test Case 1 : all is ok
         */
        // Initiliaze Mocks
        expect(deploymentManager.getServiceUnitName()).andReturn(suName);
        try {
            serviceAssemblyLifeCycle.registerSUIntoSA(suName, serviceUnitLifeCycle);
        } catch (JBIException e1) {
            e1.printStackTrace();
        }
        replay(serviceAssemblyLifeCycle);
        replay(serviceUnitLifeCycle);
        replay(deploymentManager);
        deploymentManager.saLifeCycle = serviceAssemblyLifeCycle;
        
        // Run Test Case 1
        try {
            deploymentManager.registerSULifeCycle(serviceUnitLifeCycle);
        } catch (ManagementException e) {
            e.printStackTrace();
        }
        verify(serviceAssemblyLifeCycle);
    }
    
    

}
