/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SharedLibray.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbicommon.descriptor;

import java.util.List;


/**
 * Shared Library production is used to describe a shared library installation
 * package.
 * 
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class SharedLibrary {

    /**
     * Shared Library identification data.
     */
    private Identification identification;

    /**
     * Shared Library class loading delegation model: <code>"self-first"</code>
     * or <code>"parent-first"</code>.
     */
    private String classLoaderDelegation;

    /**
     * Optional shared library version.
     */
    private String version;

    /**
     * Shared Library Class Path elements.
     */
    private List<String> sharedLibraryClassPath;

    /**
     * Default constructor.
     */
    public SharedLibrary() {
        super();
    }

    //
    // Getters and Setters
    //

    /**
     * Returns the class loading delegation model of this Shared Library. If
     * <code>null</code> the "parent-first" default delegation model must be
     * used.
     * 
     * @return <code>String</code> the delegation model:
     *         <code>"parent-first"</code> or <code>"self-first"</code>.
     */
    public String getClassLoaderDelegation() {
        return classLoaderDelegation;
    }

    /**
     * Returns component's identification data.
     * 
     * @return <code>Identification</code> the component's identification.
     */
    public Identification getIdentification() {
        return identification;
    }

    /**
     * @return Returns the sharedLibraryClassPath.
     */
    public List<String> getSharedLibraryClassPath() {
        return sharedLibraryClassPath;
    }

    /**
     * @return Returns the version.
     */
    public String getVersion() {
        return version;
    }

    /**
     * Set the Shared Library class loading delegation model.
     * 
     * @param delegationModel
     *            the delegation model to set: <code>"parent-first"</code> or
     *            <code>"self-first"</code>.
     */
    protected void setClassLoaderDelegation(String delegationModel) {
        this.classLoaderDelegation = delegationModel;
    }

    /**
     * Set the component identification data.
     * 
     * @param identification
     *            The identification to set.
     */
    protected void setIdentification(Identification identification) {
        this.identification = identification;
    }

    /**
     * @param sharedLibraryClassPath
     *            The sharedLibraryClassPath to set.
     */
    protected void setSharedLibraryClassPath(List<String> sharedLibraryClassPath) {
        this.sharedLibraryClassPath = sharedLibraryClassPath;
    }

    /**
     * @param version
     *            The version to set.
     */
    protected void setVersion(String version) {
        this.version = version;
    }
}